// complex.cpp
// Example code for operator overloading.
// M. Schukat, 2002


// Include header files
#include <iostream.h>


// Class complexNumber
class complexNumber
{
private:
	// A complex number consists of a real and an imaginary part
	double realPart, imaginaryPart;

public:
	// This is the constructor 
	complexNumber(double r = 0, double i = 0) : realPart(r), imaginaryPart(i) {};  

	// Implementation of binary operator +
	// Syntax: <ClassName> operator+(const <ClassName>& <VarName>) const
	// Function returns complexNumber
	complexNumber operator+(const complexNumber& c) const
	{
		complexNumber result;

		// Do the addition
		result.realPart = realPart + c.realPart;
		result.imaginaryPart = imaginaryPart + c.imaginaryPart;

		return result;
	}

	// Prototype for binary operator *
	// Syntax: <ClassName> operator*(const <ClassName>& <VarName>) const;
	// Function returns complexNumber
	complexNumber operator*(const complexNumber& c) const;


	// Implementation of unary operator ++ in praefix position (e.g. ++X) 
	// Remark: A prefix operator returns a reference, since ++X is a l-value
	// Syntax: <ClassName>& operator++(void)
	complexNumber& operator++(void)
	{
		realPart++;
		imaginaryPart++;

		return *this;
	}


	// Implementation of unary operator ++ in postfix position (e.g. X++) 
	// Syntax: <ClassName> operator++(int)
	// Remark: A prefix operator returns a copy, since X++ is a r-value
	//                        :--- (int) is used to determine postfix
	complexNumber operator++(int)
	{
		realPart++;
		imaginaryPart++;

		return *this;
	}

	
	// Prototype for unary operator -- in praefix position (e.g. --X) 
	// Remark: A prefix operator returns a reference, since --X is a l-value
	// Syntax: <ClassName>& operator--(void);
	complexNumber& operator--(void);


	// Prototype of unary operator -- in postfix position (e.g. X--) 
	// Remark: A prefix operator returns a copy, since X-- is a r-value
	//                        :--- (int) is used to determine postfix
	// Syntax: <ClassName> operator--(int)
	complexNumber operator--(int);

	
	// Overloading of the << operator for I/O streams.
	// Remark: This must be declared as a friend function
	// Syntax: friend ostream& operator<<(ostream& <VarName>, const <ClassName>& <VarName>)
	friend ostream& operator<<(ostream& os, const complexNumber& x)
	{
		os << x.realPart << " + " << x.imaginaryPart << "i";

		return os;
	}

};


// Implementation of complexNumber operator*(const complexNumber& c) const;
// Syntax: <ClassName> <ClassName>::operator*(const <ClassName>& <VarName>) const
complexNumber complexNumber::operator*(const complexNumber& c) const
{
	complexNumber result;

	// (a + bi) (c + di) = (ac - bd) + (bc + ad) i 
	result.realPart = (realPart * c.realPart) - (imaginaryPart * c.imaginaryPart);
	result.imaginaryPart = (imaginaryPart * c.realPart) + (realPart * c.imaginaryPart);

	return result;
}


// Implementation of unary operator -- in praefix position (e.g. --X) 
// Remark: A prefix operator returns a reference, since --X is a l-value
// Syntax: <ClassName>& <ClassName>::operator--(void)
complexNumber& complexNumber::operator--(void)
{
	realPart--;
	imaginaryPart--;

	return *this;
}


// Implementation of unary operator -- in postfix position (e.g. X--) 
// Syntax: <ClassName> <ClassName>::operator--(int)
// Remark: A prefix operator returns a copy, since X-- is a r-value
//                        :--- (int) is used to determine postfix
complexNumber complexNumber::operator--(int)
{
	realPart--;
	imaginaryPart--;

	return *this;
}

	
// The main function
int main(void)
{
	// We define a couple of complex numbers
	complexNumber a(2, 2);
	complexNumber b(3, 4);
	complexNumber c, d;
	complexNumber e(9, 8);
	complexNumber f(7, 3);

	// Print the numbers
	cout << "a(2, 2) = " << a << endl;
	cout << "b(3, 4) = " << b << endl;
	cout << "e(9, 8) = " << e << endl;
	cout << "f(7, 3) = " << f << endl << endl;

	// Test the overloaded binary operators + and *
	c = a + b;
	d = a * b;

	// Print the results
	cout << "a + b = " << c << endl;
	cout << "a * b = " << d << endl << endl;

	// Test the overloaded unary operators ++X and X++
	a++;
	++b;

	// Print the results
	cout << "a++ = " << a << endl;
	cout << "++b = " << b << endl << endl;

	// Test the overloaded unary operators --X and X--
	e--;
	--f;

	// Print the results
	cout << "e-- = " << e << endl;
	cout << "--f = " << f << endl << endl;

	return 0;
}

